@@ -11,6 +11,7 @@
 #include <complex.h>
 #include <stdio.h>
 
+#include "num/init.h"
 #include "num/flpmath.h"
 
 #include "misc/mmio.h"
@@ -28,6 +29,8 @@ int main_cabs(int argc, char* argv[])
 {
 	mini_cmdline(&argc, argv, 2, usage_str, help_str);
 
+	num_init();
+
 	long dims[DIMS];
 
 	complex float* idata = load_cfl(argv[1], DIMS, dims);
