@@ -1,11 +1,11 @@
-/* Copyright 2014-2015. The Regents of the University of California.
+/* Copyright 2014-2017. The Regents of the University of California.
  * Copyright 2015-2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
  * 2013-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
- * 2015 Jonathan Tamir <jtamir@eecs.berkeley.edu>
+ * 2015, 2017 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  */
 
 #include <stdbool.h>
@@ -20,6 +20,7 @@
 #include "misc/mmio.h"
 #include "misc/misc.h"
 #include "misc/opts.h"
+#include "misc/nested.h"
 
 
 
@@ -34,13 +35,10 @@ static const char help_str[] = "Perform homodyne reconstruction along dimension
 struct wdata {
 
 	float frac;
-	float alpha;
 	int pfdim;
 	long wdims[DIMS];
 	long wstrs[DIMS];
 	complex float* weights;
-
-	bool clear;
 };
 
 
@@ -74,15 +72,10 @@ static float homodyne_filter(long N, float frac, float alpha, bool clear, long p
 	return ret;
 }
 
-static void comp_weights(void* _data, const long pos[])
-{
-	struct wdata* data = _data;
-	data->weights[md_calc_offset(DIMS, data->wstrs, pos) / CFL_SIZE] 
-		= homodyne_filter(data->wdims[data->pfdim], data->frac, data->alpha, data->clear, pos[data->pfdim]);
-}
+
 
 static complex float* estimate_phase(struct wdata wdata, unsigned int flags,
-		unsigned int N, const long dims[N], const complex float* idata)
+		unsigned int N, const long dims[N], const complex float* idata, bool center_fft)
 {
 
 	long cdims[N];
@@ -97,7 +90,7 @@ static complex float* estimate_phase(struct wdata wdata, unsigned int flags,
 	md_resize_center(N, dims, phase, cdims, center, CFL_SIZE);
 	md_free(center);
 
-	ifftuc(N, dims, flags, phase, phase);
+	(center_fft ? ifftuc : ifftu)(N, dims, flags, phase, phase);
 	md_zphsr(N, dims, phase, phase);
 
 	return phase;
@@ -105,10 +98,10 @@ static complex float* estimate_phase(struct wdata wdata, unsigned int flags,
 
 static void homodyne(struct wdata wdata, unsigned int flags, unsigned int N, const long dims[N],
 		const long strs[N], complex float* data, const complex float* idata,
-		const long pstrs[N], const complex float* phase)
+		const long pstrs[N], const complex float* phase, bool center_fft)
 {
 	md_zmul2(N, dims, strs, data, strs, idata, wdata.wstrs, wdata.weights);
-	ifftuc(N, dims, flags, data, data);
+	(center_fft ? ifftuc : ifftu)(N, dims, flags, data, data);
 
 	md_zmulc2(N, dims, strs, data, strs, data, pstrs, phase);
 	md_zreal(N, dims, data, data);
@@ -120,6 +113,7 @@ int main_homodyne(int argc, char* argv[])
 {
 	bool clear = false;
 	bool image = false;
+	bool center_fft = true;
 	const char* phase_ref = NULL;
 
 	float alpha = 0.;
@@ -132,6 +126,7 @@ int main_homodyne(int argc, char* argv[])
 		OPT_SET('I', &image, "Input is in image domain"),
 		OPT_SET('C', &clear, "Clear unacquired portion of kspace"),
 		OPT_STRING('P', &phase_ref, "phase_ref>", "Use <phase_ref> as phase reference"),
+		OPT_CLEAR('n', &center_fft, "use uncentered ffts"),
 	};
 
 	cmdline(&argc, argv, 4, 4, usage_str, help_str, ARRAY_SIZE(opts), opts);
@@ -150,7 +145,7 @@ int main_homodyne(int argc, char* argv[])
 
 	if (image) {
 		complex float* ksp_in = md_alloc(N, dims, CFL_SIZE);
-		fftuc(N, dims, FFT_FLAGS, ksp_in, idata);
+		(center_fft ? fftuc : fftu)(N, dims, FFT_FLAGS, ksp_in, idata);
 		md_copy(N, dims, idata, ksp_in, CFL_SIZE);
 		md_free(ksp_in);
 	}
@@ -165,10 +160,14 @@ int main_homodyne(int argc, char* argv[])
 	md_select_dims(N, MD_BIT(pfdim), wdata.wdims, dims);
 	md_calc_strides(N, wdata.wstrs, wdata.wdims, CFL_SIZE);
 	wdata.weights = md_alloc(N, wdata.wdims, CFL_SIZE);
-	wdata.alpha = alpha;
-	wdata.clear = clear;
 
-	md_loop(N, wdata.wdims, &wdata, comp_weights);
+	NESTED(void, comp_weights, (const long pos[]))
+	{
+		wdata.weights[md_calc_offset(DIMS, wdata.wstrs, pos) / CFL_SIZE]
+			= homodyne_filter(wdata.wdims[pfdim], frac, alpha, clear, pos[pfdim]);
+	};
+
+	md_loop(N, wdata.wdims, comp_weights);
 
 	long pstrs[N];
 	long pdims[N];
@@ -176,7 +175,7 @@ int main_homodyne(int argc, char* argv[])
 
 	if (NULL == phase_ref) {
 
-		phase = estimate_phase(wdata, FFT_FLAGS, N, dims, idata);
+		phase = estimate_phase(wdata, FFT_FLAGS, N, dims, idata, center_fft);
 		md_copy_dims(N, pdims, dims);
 	}
 	else
@@ -184,7 +183,7 @@ int main_homodyne(int argc, char* argv[])
 
 	md_calc_strides(N, pstrs, pdims, CFL_SIZE);
 
-	homodyne(wdata, FFT_FLAGS, N, dims, strs, data, idata, pstrs, phase);
+	homodyne(wdata, FFT_FLAGS, N, dims, strs, data, idata, pstrs, phase, center_fft);
 
 	md_free(wdata.weights);
 
