@@ -16,6 +16,13 @@ typedef void iter3_fun_f(iter3_conf* _conf,
 		long N, float* dst, const float* ref,
 		long M, const float* src);
 
+typedef void iter3_irgnm_f(iter3_conf* _conf,
+		struct iter_op_s frw,
+		struct iter_op_s der,
+		struct iter_op_s adj,
+		long N, float* dst, const float* ref,
+		long M, const float* src,
+		struct iter_op_s cb);
 
 
 struct iter3_irgnm_conf {
@@ -34,7 +41,7 @@ struct iter3_irgnm_conf {
 
 extern DEF_TYPEID(iter3_irgnm_conf);
 
-iter3_fun_f iter3_irgnm;
+iter3_irgnm_f iter3_irgnm;
 
 
 
