@@ -16,7 +16,7 @@ struct operator_p_s;
 
 
 // Low rank thresholding for arbitrary block sizes
-extern const struct operator_p_s* lrthresh_create(const long dims_lev[DIMS], _Bool randshift, unsigned long mflags, const long blkdims[MAX_LEV][DIMS], float lambda, _Bool noise, int remove_mean, _Bool use_gpu);
+extern const struct operator_p_s* lrthresh_create(const long dims_lev[DIMS], _Bool randshift, unsigned long mflags, const long blkdims[MAX_LEV][DIMS], float lambda, _Bool noise, int remove_mean);
 
 // Returns nuclear norm using lrthresh operator
 extern float lrnucnorm(const struct operator_p_s* op, const complex float* src);
