@@ -16,6 +16,13 @@
 
 
 #define CAST_CONST(T, x)  ((T)TYPE_CHECK(const T, x))
+#define CAST_MAYBE(T, x)	({ \
+	__typeof__(x) __tmp = (x); \
+	extern __typeof__(*__tmp->TYPEID) T ## _TYPEID; \
+	(__tmp->TYPEID == &T ## _TYPEID) ?		\
+		CONTAINER_OF(__tmp, struct T, INTERFACE)\
+		: NULL;	\
+})
 #define CAST_DOWN(T, x)	({ \
 	__typeof__(x) __tmp = (x); \
 	extern __typeof__(*__tmp->TYPEID) T ## _TYPEID; \
@@ -27,7 +34,7 @@
 
 #define INTERFACE(X) X INTERFACE
 
-typedef const struct typeid_s { int:0; } TYPEID;
+typedef const struct typeid_s { int:1; } TYPEID;
 
 #define TYPEID(T) T ## _TYPEID
 #define DEF_TYPEID(T) TYPEID TYPEID(T)
