@@ -22,6 +22,9 @@ struct noir_model_conf_s {
 	_Bool rvc;
 	_Bool use_gpu;
 	_Bool noncart;
+	_Bool pattern_for_each_coil;
+	float a;
+	float b;
 };
 
 extern struct noir_model_conf_s noir_model_conf_defaults;
@@ -29,5 +32,7 @@ extern struct noir_model_conf_s noir_model_conf_defaults;
 extern struct noir_data* noir_init(const long dims[DIMS], const complex float* mask, const complex float* psf, const struct noir_model_conf_s* conf);
 extern void noir_free(struct noir_data* data);
 
+extern void noir_orthogonalize(struct noir_data*, complex float* dst, const complex float* src);
+
 
 
