@@ -37,6 +37,17 @@
 #include "recon.h"
 
 
+struct nlop_wrapper_s {
+	INTERFACE(struct iter_op_data_s);
+	struct nlop_s* nlop;
+};
+DEF_TYPEID(nlop_wrapper_s);
+
+
+static void orthogonalize(iter_op_data* ptr, float* _dst, const float* _src)
+{
+	noir_orthogonalize(noir_get_data(CAST_DOWN(nlop_wrapper_s, ptr)->nlop), (complex float*)_dst, (const complex float*)_src);
+}
 
 const struct noir_conf_s noir_defaults = {
 
@@ -46,10 +57,12 @@ const struct noir_conf_s noir_defaults = {
 	.noncart = false,
 	.alpha = 1.,
 	.redu = 2.,
+	.a = 220.,
+	.b = 32.,
+	.pattern_for_each_coil = false,
 };
 
-
-void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex float* outbuf, complex float* sensout, const complex float* psf, const complex float* mask, const complex float* kspace)
+void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex float* img, complex float* sens, const complex float* pattern, const complex float* mask, const complex float* kspace_data )
 {
 	long imgs_dims[DIMS];
 	long coil_dims[DIMS];
@@ -58,7 +71,7 @@ void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex f
 
 	unsigned int fft_flags = FFT_FLAGS|SLICE_FLAG;
 
-	md_select_dims(DIMS, fft_flags|MAPS_FLAG|CSHIFT_FLAG, imgs_dims, dims);
+	md_select_dims(DIMS, fft_flags|MAPS_FLAG, imgs_dims, dims);
 	md_select_dims(DIMS, fft_flags|COIL_FLAG|MAPS_FLAG, coil_dims, dims);
 	md_select_dims(DIMS, fft_flags|COIL_FLAG, data_dims, dims);
 	md_select_dims(DIMS, fft_flags, img1_dims, dims);
@@ -68,23 +81,22 @@ void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex f
 	long data_size = md_calc_size(DIMS, data_dims);
 
 	long d1[1] = { size };
-	complex float* img = md_alloc_sameplace(1, d1, CFL_SIZE, kspace);
+	// variable which is optimized by the IRGNM
+	complex float* x = md_alloc_sameplace(1, d1, CFL_SIZE, kspace_data );
 
-
-	md_clear(DIMS, imgs_dims, img, CFL_SIZE);
-
-	md_zfill(DIMS, img1_dims, outbuf, 1.);	// initial only first image
-	md_copy(DIMS, img1_dims, img, outbuf, CFL_SIZE);
-
-	md_clear(DIMS, coil_dims, img + skip, CFL_SIZE);
+	md_copy(DIMS, imgs_dims, x, img, CFL_SIZE);
+	md_copy(DIMS, coil_dims, x + skip, sens, CFL_SIZE);
 
 	struct noir_model_conf_s mconf = noir_model_conf_defaults;
 	mconf.rvc = conf->rvc;
 	mconf.use_gpu = conf->usegpu;
 	mconf.noncart = conf->noncart;
 	mconf.fft_flags = fft_flags;
+	mconf.pattern_for_each_coil = conf->pattern_for_each_coil;
+	mconf.a = conf->a;
+	mconf.b = conf->b;
 
-	struct nlop_s* nlop = noir_create(dims, mask, psf, &mconf);
+	struct nlop_s* nlop = noir_create(dims, mask, pattern, &mconf);
 
 	struct iter3_irgnm_conf irgnm_conf = iter3_irgnm_defaults;
 
@@ -94,23 +106,35 @@ void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], complex f
 	irgnm_conf.cgtol = 0.1f;
 	irgnm_conf.nlinv_legacy = true;
 
+	struct nlop_wrapper_s nlw;
+	SET_TYPEID(nlop_wrapper_s, &nlw);
+	nlw.nlop = nlop;
+
+
 	iter4_irgnm(CAST_UP(&irgnm_conf),
 			nlop,
-			size * 2, (float*)img, NULL,
-			data_size * 2, (const float*)kspace);
+			size * 2, (float*) x, NULL,
+			data_size * 2, (const float*) kspace_data,
+			(struct iter_op_s){ orthogonalize, CAST_UP(&nlw) });
 
-	md_copy(DIMS, imgs_dims, outbuf, img, CFL_SIZE);
+	md_copy(DIMS, imgs_dims, img, x, CFL_SIZE);
 
-	if (NULL != sensout) {
+	if (NULL != sens ) {
 
-		assert(!conf->usegpu);
-		noir_forw_coils(noir_get_data(nlop), sensout, img + skip);
-		fftmod(DIMS, coil_dims, fft_flags, sensout, sensout);
+#ifdef USE_CUDA
+		if (conf->usegpu) {
+
+			noir_forw_coils(noir_get_data(nlop), x + skip, x + skip);
+			md_copy(DIMS, coil_dims, sens, x + skip, CFL_SIZE);
+		} else
+#endif
+			noir_forw_coils(noir_get_data(nlop), sens, x + skip);
+		fftmod(DIMS, coil_dims, fft_flags, sens, sens);
 	}
 
 	nlop_free(nlop);
 
-	md_free(img);
+	md_free(x);
 }
 
 
