@@ -1,11 +1,11 @@
 /* Copyright 2014-2015. The Regents of the University of California.
- * Copyright 2016-2017. Martin Uecker.
+ * Copyright 2016-2018. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
  * 2014-2017 Frank Ong <frankong@berkeley.edu>
- * 2014-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2014-2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  *
  * Strang G. A proposal for Toeplitz matrix calculations. Journal Studies in Applied Math. 1986; 74(2):171-17
  *
@@ -42,6 +42,7 @@ struct nufft_conf_s nufft_conf_defaults = {
 
 	.toeplitz = true,
 	.pcycle = false,
+	.periodic = false,
 };
 
 
@@ -480,6 +481,7 @@ complex float* compute_psf(unsigned int N, const long img2_dims[N], const long t
 	md_select_dims(N, ~MD_BIT(0), ksp_dims1, trj_dims);
 
 	struct nufft_conf_s conf = nufft_conf_defaults;
+	conf.periodic = true; // FIXME: periodic ???
 	conf.toeplitz = false;	// avoid infinite loop
 
 	struct linop_s* op2 = nufft_create(N, ksp_dims1, img2_dims, trj_dims, traj, NULL, conf);
@@ -563,32 +565,32 @@ static void nufft_free_data(const linop_data_t* _data)
 {
 	struct nufft_data* data = CAST_DOWN(nufft_data, _data);
 
-	free(data->ksp_dims);
-	free(data->cim_dims);
-	free(data->cml_dims);
-	free(data->img_dims);
-	free(data->trj_dims);
-	free(data->lph_dims);
-	free(data->psf_dims);
-	free(data->wgh_dims);
+	xfree(data->ksp_dims);
+	xfree(data->cim_dims);
+	xfree(data->cml_dims);
+	xfree(data->img_dims);
+	xfree(data->trj_dims);
+	xfree(data->lph_dims);
+	xfree(data->psf_dims);
+	xfree(data->wgh_dims);
 
-	free(data->ksp_strs);
-	free(data->cim_strs);
-	free(data->cml_strs);
-	free(data->img_strs);
-	free(data->trj_strs);
-	free(data->lph_strs);
-	free(data->psf_strs);
-	free(data->wgh_strs);
+	xfree(data->ksp_strs);
+	xfree(data->cim_strs);
+	xfree(data->cml_strs);
+	xfree(data->img_strs);
+	xfree(data->trj_strs);
+	xfree(data->lph_strs);
+	xfree(data->psf_strs);
+	xfree(data->wgh_strs);
 
-	free(data->rlph_dims);
-	free(data->rpsf_dims);
-	free(data->rcml_dims);
-	free(data->rlph_strs);
-	free(data->rpsf_strs);
-	free(data->rcml_strs);
+	xfree(data->rlph_dims);
+	xfree(data->rpsf_dims);
+	xfree(data->rcml_dims);
+	xfree(data->rlph_strs);
+	xfree(data->rpsf_strs);
+	xfree(data->rcml_strs);
 
-	free(data->cm2_dims);
+	xfree(data->cm2_dims);
 
 	md_free(data->grid);
 	md_free(data->linphase);
@@ -606,7 +608,7 @@ static void nufft_free_data(const linop_data_t* _data)
 	if (data->conf.pcycle)
 		linop_free(data->cfft_op);
 
-	free(data);
+	xfree(data);
 }
 
 
@@ -639,7 +641,15 @@ static void nufft_apply(const linop_data_t* _data, complex float* dst, const com
 
 	md_recompose(data->N, factors, data->cm2_dims, gridX, data->cml_dims, data->grid, CFL_SIZE);
 
-	grid2H(2., data->width, data->beta, ND, data->trj_dims, data->traj, data->ksp_dims, dst, data->cm2_dims, gridX);
+	struct grid_conf_s conf = {
+
+		.width = data->width,
+		.os = 2.,
+		.periodic = data->conf.periodic,
+		.beta = data->beta,
+	};
+
+	grid2H(&conf, ND, data->trj_dims, data->traj, data->ksp_dims, dst, data->cm2_dims, gridX);
 
 	md_free(gridX);
 
@@ -669,7 +679,15 @@ static void nufft_apply_adjoint(const linop_data_t* _data, complex float* dst, c
 		src = wdat;
 	}
 
-	grid2(2., data->width, data->beta, ND, data->trj_dims, data->traj, data->cm2_dims, gridX, data->ksp_dims, src);
+	struct grid_conf_s conf = {
+
+		.width = data->width,
+		.os = 2.,
+		.periodic = data->conf.periodic,
+		.beta = data->beta,
+	};
+
+	grid2(&conf, ND, data->trj_dims, data->traj, data->cm2_dims, gridX, data->ksp_dims, src);
 
 	md_free(wdat);
 
