@@ -42,6 +42,7 @@
 #include "misc/misc.h"
 #include "misc/types.h"
 #include "misc/debug.h"
+#include "misc/nested.h"
 
 #ifdef USE_CUDA
 #include "num/gpuops.h"
@@ -104,16 +105,15 @@ static void make_2op_simple(md_2op_t fun, unsigned int D, const long dims[D], fl
  * @param iptr1 input 1 (constant)
  * @param size size of data structures, e.g. complex float 
  * @param too two-op multiply function
- * @param data_ptr pointer to additional data used by too
  */
-static void optimized_twoop_oi(unsigned int D, const long dim[D], const long ostr[D], void* optr, const long istr1[D], const void* iptr1, size_t sizes[2], md_nary_opt_fun_t too, void* data_ptr)
+static void optimized_twoop_oi(unsigned int D, const long dim[D], const long ostr[D], void* optr, const long istr1[D], const void* iptr1, size_t sizes[2], md_nary_opt_fun_t too)
 {
 	const long (*nstr[2])[D] = { (const long (*)[D])ostr, (const long (*)[D])istr1 };
 	void *nptr[2] = { optr, (void*)iptr1 };
 
 	unsigned int io = 1 + ((iptr1 == optr) ? 2 : 0);
 
-	optimized_nop(2, io, D, dim, nstr, nptr, sizes, too, data_ptr);
+	optimized_nop(2, io, D, dim, nstr, nptr, sizes, too);
 }
 
 
@@ -134,16 +134,15 @@ static void optimized_twoop_oi(unsigned int D, const long dim[D], const long ost
  * @param iptr2 input 2 (constant)
  * @param size size of data structures, e.g. complex float 
  * @param too three-op multiply function
- * @param data_ptr pointer to additional data used by too
  */
-static void optimized_threeop_oii(unsigned int D, const long dim[D], const long ostr[D], void* optr, const long istr1[D], const void* iptr1, const long istr2[D], const void* iptr2, size_t sizes[3], md_nary_opt_fun_t too, void* data_ptr)
+static void optimized_threeop_oii(unsigned int D, const long dim[D], const long ostr[D], void* optr, const long istr1[D], const void* iptr1, const long istr2[D], const void* iptr2, size_t sizes[3], md_nary_opt_fun_t too)
 {
 	const long (*nstr[3])[D] = { (const long (*)[D])ostr, (const long (*)[D])istr1, (const long (*)[D])istr2 };
 	void *nptr[3] = { optr, (void*)iptr1, (void*)iptr2 };
 
 	unsigned int io = 1 + ((iptr1 == optr) ? 2 : 0) + ((iptr2 == optr) ? 4 : 0);
 
-	optimized_nop(3, io, D, dim, nstr, nptr, sizes, too, data_ptr);
+	optimized_nop(3, io, D, dim, nstr, nptr, sizes, too);
 }
 
 
@@ -245,87 +244,69 @@ static void make_2opd_simple(md_2opd_t fun, unsigned int D, const long dims[D],
 	fun(D, dims, strs_double, optr, strs_single, iptr1);
 }
 
-static void nary_z3op(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(z3op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
-}
-
 static void make_z3op(size_t offset, unsigned int D, const long dim[D], const long ostr[D], complex float* optr, const long istr1[D], const complex float* iptr1, const long istr2[D], const complex float* iptr2)
 {
+	NESTED(void, nary_z3op, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(z3op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
+	};
+
 	optimized_threeop_oii(D, dim, ostr, optr, istr1, iptr1, istr2, iptr2,
-				(size_t[3]){ [0 ... 2] = CFL_SIZE }, nary_z3op, &offset);
+				(size_t[3]){ [0 ... 2] = CFL_SIZE }, nary_z3op);
 }
 
-static void nary_3op(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(r3op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
-}
 
 static void make_3op(size_t offset, unsigned int D, const long dim[D], const long ostr[D], float* optr, const long istr1[D], const float* iptr1, const long istr2[D], const float* iptr2)
 {
+	NESTED(void, nary_3op, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(r3op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
+	};
+
 	optimized_threeop_oii(D, dim, ostr, optr, istr1, iptr1, istr2, iptr2,
-				(size_t[3]){ [0 ... 2] = FL_SIZE }, nary_3op, &offset);
-}
-
-static void nary_z3opd(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(z3opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
+				(size_t[3]){ [0 ... 2] = FL_SIZE }, nary_3op);
 }
 
 static void make_z3opd(size_t offset, unsigned int D, const long dim[D], const long ostr[D], complex double* optr, const long istr1[D], const complex float* iptr1, const long istr2[D], const complex float* iptr2)
 {
+	NESTED(void, nary_z3opd, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(z3opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
+	};
+
 	optimized_threeop_oii(D, dim, ostr, optr, istr1, iptr1, istr2, iptr2,
-			(size_t[3]){ CDL_SIZE, CFL_SIZE, CFL_SIZE }, nary_z3opd, &offset);
-}
-
-static void nary_3opd(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(r3opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
+			(size_t[3]){ CDL_SIZE, CFL_SIZE, CFL_SIZE }, nary_z3opd);
 }
 
 static void make_3opd(size_t offset, unsigned int D, const long dim[D], const long ostr[D], double* optr, const long istr1[D], const float* iptr1, const long istr2[D], const float* iptr2)
 {
+	NESTED(void, nary_3opd, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(r3opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1], ptr[2]);
+	};
+
 	optimized_threeop_oii(D, dim, ostr, optr, istr1, iptr1, istr2, iptr2,
-			(size_t[3]){ DL_SIZE, FL_SIZE, FL_SIZE }, nary_3opd, &offset);
-}
-
-static void nary_z2op(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(z2op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+			(size_t[3]){ DL_SIZE, FL_SIZE, FL_SIZE }, nary_3opd);
 }
 
 static void make_z2op(size_t offset, unsigned int D, const long dim[D], const long ostr[D], complex float* optr, const long istr1[D], const complex float* iptr1)
 {
-	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_z2op, &offset);
-}
+	NESTED(void, nary_z2op, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(z2op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	};
 
-static void nary_2op(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(r2op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_z2op);
 }
 
 static void make_2op(size_t offset, unsigned int D, const long dim[D], const long ostr[D], float* optr, const long istr1[D], const float* iptr1)
 {
-	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ FL_SIZE, FL_SIZE }, nary_2op, &offset);
-}
+	NESTED(void, nary_2op, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(r2op_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	};
 
-static void nary_z2opd(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(z2opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ FL_SIZE, FL_SIZE }, nary_2op);
 }
 
 __attribute__((unused))
@@ -333,47 +314,47 @@ static void make_z2opd(size_t offset, unsigned int D, const long dim[D], const l
 {
 	size_t sizes[2] = { sizeof(complex double), sizeof(complex float) };
 
-	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, sizes, nary_z2opd, &offset);
+	NESTED(void, nary_z2opd, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(z2opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, sizes, nary_z2opd);
 }
 
 
-static void nary_2opd(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(r2opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
-}
-
 static void make_2opd(size_t offset, unsigned int D, const long dim[D], const long ostr[D], double* optr, const long istr1[D], const float* iptr1)
 {
-	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ DL_SIZE, FL_SIZE }, nary_2opd, &offset);
-}
+	NESTED(void, nary_2opd, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(r2opd_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	};
 
-static void nary_z2opf(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(z2opf_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ DL_SIZE, FL_SIZE }, nary_2opd);
 }
 
 static void make_z2opf(size_t offset, unsigned int D, const long dim[D], const long ostr[D], complex float* optr, const long istr1[D], const complex double* iptr1)
 {
 	size_t sizes[2] = { sizeof(complex float), sizeof(complex double) };
-	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, sizes, nary_z2opf, &offset);
+
+	NESTED(void, nary_z2opf, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(z2opf_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, sizes, nary_z2opf);
 }
 
 void* unused2 = make_z2opf;
 
-static void nary_2opf(struct nary_opt_data_s* data, void* ptr[])
-{
-	size_t offset = *(size_t*)data->data_ptr;
-
-	(*(r2opf_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
-}
-
 static void make_2opf(size_t offset, unsigned int D, const long dim[D], const long ostr[D], float* optr, const long istr1[D], const double* iptr1)
 {
-	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ FL_SIZE, DL_SIZE }, nary_2opf, &offset);
+	NESTED(void, nary_2opf, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		(*(r2opf_t*)(((char*)data->ops) + offset))(data->size, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dim, ostr, optr, istr1, iptr1, (size_t[2]){ FL_SIZE, DL_SIZE }, nary_2opf);
 }
 
 static void make_z2opf_simple(md_z2opf_t fun, unsigned int D, const long dims[D], complex float* optr, const complex double* iptr1)
@@ -499,7 +480,7 @@ static void make_z2opf_from_real(size_t offset, unsigned int D, const long dims[
 }
 
 
-// type save
+// type safe
 #define MAKE_3OP(fun, ...)	((void)TYPE_CHECK(r3op_t, cpu_ops.fun), make_3op(offsetof(struct vec_ops, fun),  __VA_ARGS__))
 #define MAKE_Z3OP(fun, ...)	((void)TYPE_CHECK(z3op_t, cpu_ops.fun), make_z3op(offsetof(struct vec_ops, fun),  __VA_ARGS__))
 #define MAKE_2OP(fun, ...)	((void)TYPE_CHECK(r2op_t, cpu_ops.fun), make_2op(offsetof(struct vec_ops, fun),  __VA_ARGS__))
@@ -626,7 +607,20 @@ void md_zsmul2(unsigned int D, const long dims[D], const long ostr[D], complex f
 		return;
 	}
 
+#if 0
 	make_z3op_scalar(md_zmul2, D, dims, ostr, optr, istr, iptr, val);
+#else
+	// FIXME: we should rather optimize md_zmul2 for this case
+
+	NESTED(void, nary_zsmul, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zsmul(data->size, val, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dims, ostr, optr, istr, iptr,
+		(size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zsmul);
+#endif
+
 }
 
 
@@ -681,7 +675,22 @@ void md_smul2(unsigned int D, const long dims[D], const long ostr[D], float* opt
 	}
 fallback:
 #endif
+
+#if 0
 	make_3op_scalar(md_mul2, D, dims, ostr, optr, istr, iptr, var);
+#else
+	// FIXME: we should rather optimize md_mul2 for this case
+
+	(void)0;
+
+	NESTED(void, nary_smul, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->smul(data->size, var, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dims, ostr, optr, istr, iptr,
+		(size_t[2]){ FL_SIZE, FL_SIZE }, nary_smul);
+#endif
 }
 
 
@@ -1808,7 +1817,7 @@ void md_slessequal(unsigned int D, const long dims[D], float* optr, const float*
  */
 void md_greatequal2(unsigned int D, const long dims[D], const long ostr[D], float* optr, const long istr1[D], const float* iptr1, const long istr2[D], const float* iptr2)
 {
-	MAKE_3OP(ge, D, dims, ostr, optr, istr1, iptr1, istr2, iptr2);
+	md_lessequal2(D, dims, ostr, optr, istr2, iptr2, istr1, iptr1);
 }
 
 
@@ -2234,6 +2243,9 @@ void md_abs(unsigned int D, const long dims[D], float* optr, const float* iptr)
 void md_zabs2(unsigned int D, const long dims[D], const long ostr[D], complex float* optr,
 		const long istr[D], const complex float* iptr)
 {
+#if 1
+	MAKE_Z2OP(zabs, D, dims, ostr, optr, istr, iptr);
+#else
 	// FIXME: special case of md_rss
 
 	assert(optr != iptr);
@@ -2252,6 +2264,7 @@ void md_zabs2(unsigned int D, const long dims[D], const long ostr[D], complex fl
 #else
 	md_zsqrt2(D, dims, ostr, optr, ostr, optr);
 #endif
+#endif
 }
 
 
@@ -2718,15 +2731,8 @@ extern void md_zfill(unsigned int D, const long dim[D], complex float* ptr, comp
 
 
 
-/**
- * Soft Thresholding complex array
- *
- * return SoftThresh(ptr)
- */
-static void nary_zsoftthresh_half(struct nary_opt_data_s* data, void* ptr[])
-{
-	data->ops->zsoftthresh_half(data->size, *(float*)data->data_ptr, ptr[0], ptr[1]);
-}
+
+
 
 /**
  * Step (2) of Soft Thresholding multi-dimensional arrays, y = ST(x, lambda)
@@ -2742,21 +2748,16 @@ static void nary_zsoftthresh_half(struct nary_opt_data_s* data, void* ptr[])
  */
 void md_zsoftthresh_half2(unsigned int D, const long dim[D], float lambda, const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr)
 {
-	optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zsoftthresh_half, &lambda);
+	NESTED(void, nary_zsoftthresh_half, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zsoftthresh_half(data->size, lambda, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zsoftthresh_half);
 }
 
 
 
-/**
- * Soft Thresholding array
- *
- * return SoftThresh(ptr)
- */
-static void nary_softthresh_half(struct nary_opt_data_s* data, void* ptr[])
-{
-	data->ops->softthresh_half(data->size, *(float*)data->data_ptr, ptr[0], ptr[1]);
-}
-
 /**
  * Step (2) of Soft Thresholding multi-dimensional arrays, y = ST(x, lambda)
  * 2) computes resid = MAX( (abs(x) - lambda)/abs(x), 0 ) (with strides)
@@ -2771,7 +2772,12 @@ static void nary_softthresh_half(struct nary_opt_data_s* data, void* ptr[])
  */
 void md_softthresh_half2(unsigned int D, const long dim[D], float lambda, const long ostr[D], float* optr, const long istr[D], const float* iptr)
 {
-	optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ FL_SIZE, FL_SIZE }, nary_softthresh_half, &lambda);
+	NESTED(void, nary_softthresh_half, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->softthresh_half(data->size, lambda, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ FL_SIZE, FL_SIZE }, nary_softthresh_half);
 }
 
 
@@ -2811,16 +2817,6 @@ void md_softthresh_core2(unsigned int D, const long dims[D], float lambda, unsig
 
 
 
-/**
- * Soft Thresholding complex array
- *
- * return SoftThresh(ptr)
- */
-static void nary_softthresh(struct nary_opt_data_s* data, void* ptr[])
-{
-	data->ops->softthresh(data->size, *(float*)data->data_ptr, ptr[0], ptr[1]);
-}
-
 /**
  * Soft Thresholding for floats (with strides)
  *
@@ -2828,9 +2824,14 @@ static void nary_softthresh(struct nary_opt_data_s* data, void* ptr[])
  */
 void md_softthresh2(unsigned int D, const long dims[D], float lambda, unsigned int flags, const long ostrs[D], float* optr, const long istrs[D], const float* iptr)
 {
+	NESTED(void, nary_softthresh, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->softthresh(data->size, lambda, ptr[0], ptr[1]);
+	};
+
 	if (0 == flags) {
 
-		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ FL_SIZE, FL_SIZE }, nary_softthresh, &lambda);
+		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ FL_SIZE, FL_SIZE }, nary_softthresh);
 		return;
 	}
 
@@ -2876,10 +2877,7 @@ void md_zsoftthresh_core2(unsigned int D, const long dims[D], float lambda, unsi
 
 
 
-static void nary_zsoftthresh(struct nary_opt_data_s* data, void* ptr[])
-{
-	data->ops->zsoftthresh(data->size, *(float*)data->data_ptr, ptr[0], ptr[1]);
-}
+
 
 /**
  * Soft thresholding using norm along arbitrary dimension (with strides)
@@ -2897,9 +2895,14 @@ static void nary_zsoftthresh(struct nary_opt_data_s* data, void* ptr[])
  */
 void md_zsoftthresh2(unsigned int D, const long dims[D], float lambda, unsigned int flags, const long ostrs[D], complex float* optr, const long istrs[D], const complex float* iptr)
 {
+	NESTED(void, nary_zsoftthresh, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zsoftthresh(data->size, lambda, ptr[0], ptr[1]);
+	};
+
 	if (0 == flags) {
 
-		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zsoftthresh, &lambda);
+		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zsoftthresh);
 		return;
 	}
 
@@ -2913,6 +2916,7 @@ void md_zsoftthresh2(unsigned int D, const long dims[D], float lambda, unsigned
 	md_free(tmp_norm);
 }
 
+
 /**
  * Soft thresholding using norm along arbitrary dimension (without strides)
  *
@@ -2935,16 +2939,9 @@ void md_zsoftthresh(unsigned int D, const long dims[D], float lambda, unsigned i
 	md_zsoftthresh2(D, dims, lambda, flags, strs, optr, strs, iptr);
 }
 
-/**
- * Hard Thresholding mask complex array (nonzero support of k-largest elements)
- * Writes to the output a mask of the non-zero elements in the input
- *
- * return HardThreshSupp(ptr)
- */
-static void nary_zhardthresh_mask(struct nary_opt_data_s* data, void* ptr[])
-{
-	data->ops->zhardthresh_mask(data->size, (*(unsigned int*)data->data_ptr), ptr[0], ptr[1]);
-}
+
+
+
 
 /**
  * Produces a mask (1s and 0s) of the non-zero support of a hard thresholded input vector
@@ -2962,8 +2959,14 @@ static void nary_zhardthresh_mask(struct nary_opt_data_s* data, void* ptr[])
  */
 void md_zhardthresh_mask2(unsigned int D, const long dim[D], unsigned int k, unsigned int flags, complex float* tmp_norm, const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr)
 {
+	NESTED(void, nary_zhardthresh_mask, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zhardthresh_mask(data->size, k, ptr[0], ptr[1]);
+	};
+
 	if (0 == flags) {
-		optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask, &k);
+
+		optimized_twoop_oi(D, dim, ostr, optr, istr, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask);
 		return;
 	}
 
@@ -2974,10 +2977,11 @@ void md_zhardthresh_mask2(unsigned int D, const long dim[D], unsigned int k, uns
 	md_calc_strides(D, norm_strs, norm_dims, CFL_SIZE);
 
 	md_zrss(D, dim, flags, tmp_norm, iptr);
-	optimized_twoop_oi(D, norm_dims, norm_strs, tmp_norm, norm_strs, tmp_norm, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask, &k);
+	optimized_twoop_oi(D, norm_dims, norm_strs, tmp_norm, norm_strs, tmp_norm, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask);
 	md_copy2(D, dim, ostr, optr, norm_strs, tmp_norm, CFL_SIZE);
 }
 
+
 /**
  * Produces a mask (1s and 0s) of the non-zero support of a hard thresholded input vector
  * Multi-dimensional operation using the same strides for input and output.
@@ -3002,9 +3006,9 @@ void md_zhardthresh_mask(unsigned int D, const long dim[D], unsigned int k, unsi
 	md_zhardthresh_mask2(D, dim, k, flags, tmp_norm, str, optr, str, iptr);
 
 	md_free(tmp_norm);
-
 }
 
+
 /**
  * Joint Hard thresholding  (with strides)
  * Performs hard thresholding to the norm along dimension specified by flags
@@ -3030,19 +3034,18 @@ void md_zhardthresh_joint2(unsigned int D, const long dims[D], unsigned int k, u
 	md_calc_strides(D, norm_strs, norm_dims, CFL_SIZE);
 
 	md_zrss(D, dims, flags, tmp_norm, iptr);
-	optimized_twoop_oi(D, norm_dims, norm_strs, tmp_norm, norm_strs, tmp_norm, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask, &k);
+
+	NESTED(void, nary_zhardthresh_mask, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zhardthresh_mask(data->size, k, ptr[0], ptr[1]);
+	};
+
+	optimized_twoop_oi(D, norm_dims, norm_strs, tmp_norm, norm_strs, tmp_norm, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh_mask);
 	md_zmul2(D, dims, ostrs, optr, norm_strs, tmp_norm, istrs, iptr);
 }
 
-/**
- * Hard Thresholding complex array (select k-largest elements)
- *
- * return HardThresh(ptr)
- */
-static void nary_zhardthresh(struct nary_opt_data_s* data, void* ptr[])
-{
-	data->ops->zhardthresh(data->size, (*(unsigned int*)data->data_ptr), ptr[0], ptr[1]);
-}
+
+
 
 /**
  * Hard thresholding (with strides)
@@ -3063,9 +3066,14 @@ static void nary_zhardthresh(struct nary_opt_data_s* data, void* ptr[])
  */
 void md_zhardthresh2(unsigned int D, const long dims[D], unsigned int k, unsigned int flags, const long ostrs[D], complex float* optr, const long istrs[D], const complex float* iptr)
 {
+	NESTED(void, nary_zhardthresh, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zhardthresh(data->size, k, ptr[0], ptr[1]);
+	};
+
 	if (0 == flags) {
 
-		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh, &k);
+		optimized_twoop_oi(D, dims, ostrs, optr, istrs, iptr, (size_t[2]){ CFL_SIZE, CFL_SIZE }, nary_zhardthresh);
 		return;
 	}
 
@@ -3073,12 +3081,12 @@ void md_zhardthresh2(unsigned int D, const long dims[D], unsigned int k, unsigne
 	md_select_dims(D, ~flags, norm_dims, dims);
 
 	complex float* tmp_norm = md_alloc_sameplace(D, norm_dims, CFL_SIZE, iptr);
-
 	md_zhardthresh_joint2(D, dims, k, flags, tmp_norm, ostrs, optr, istrs,iptr);
 
 	md_free(tmp_norm);
 }
 
+
 /**
  * Hard thresholding (without strides)
  *
@@ -3097,9 +3105,9 @@ void md_zhardthresh(unsigned int D, const long dims[D], unsigned int k, unsigned
 	md_calc_strides(D, strs, dims, CFL_SIZE);
 
 	md_zhardthresh2(D, dims, k, flags, strs, optr, strs, iptr);
-
 }
 
+
 /**
  * Elementwise minimum of input and scalar (with strides)
  *
@@ -3288,20 +3296,6 @@ void md_zfdiff_backwards(unsigned int D, const long dims[D], unsigned int d, com
 
 
 
-struct zfftmod_s {
-
-	double phase;
-	bool inv;
-	unsigned int N;
-};
-
-static void nary_zfftmod(struct nary_opt_data_s* data, void* ptr[])
-{
-	struct zfftmod_s* mdata = (struct zfftmod_s*)data->data_ptr;
-
-	data->ops->zfftmod(data->size, ptr[0], ptr[1], mdata->N, mdata->inv, mdata->phase);
-}
-
 // DO NOT USE DIRECTLY - this is used internally by fftmod from fft.[ch]
 void md_zfftmod2(unsigned int D, const long dims[D], const long ostrs[D], complex float* optr, const long istrs[D], const complex float* iptr, bool inv, double phase)
 {
@@ -3310,8 +3304,13 @@ void md_zfftmod2(unsigned int D, const long dims[D], const long ostrs[D], comple
 
 	unsigned int N = dims[0];
 
+	NESTED(void, nary_zfftmod, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->zfftmod(data->size, ptr[0], ptr[1], N, inv, phase);
+	};
+
 	optimized_twoop_oi(D - 1, dims + 1, ostrs + 1, optr, istrs + 1, iptr,
-		(size_t[2]){ N * CFL_SIZE, N * CFL_SIZE }, nary_zfftmod, &(struct zfftmod_s){ phase, inv, N });
+		(size_t[2]){ N * CFL_SIZE, N * CFL_SIZE }, nary_zfftmod);
 }
 
 void md_zfftmod(unsigned int D, const long dims[D], complex float* optr, const complex float* iptr, bool inv, double phase)
