@@ -24,8 +24,6 @@
 #include <cuda.h>
 #include <cublas.h>
 
-#include <omp.h>
-
 #include "num/vecops.h"
 #include "num/gpuops.h"
 #include "num/gpukrnls.h"
@@ -256,7 +254,7 @@ void cuda_hostfree(void* ptr)
 	struct cuda_mem_s* nptr = search(ptr, true);
 	assert(nptr->ptr == ptr);
 	assert(!nptr->device);
-	free(nptr);
+	xfree(nptr);
 
 	cudaFreeHost(ptr);
 }
@@ -328,13 +326,16 @@ const struct vec_ops gpu_ops = {
 	.fmac = cuda_fmac,
 	.fmac2 = cuda_fmac2,
 
+	.smul = cuda_smul,
+
 	.axpy = cuda_saxpy,
 
 	.pow = cuda_pow,
 	.sqrt = cuda_sqrt,
 
 	.le = cuda_le,
-	.ge = cuda_ge,
+
+	.zsmul = cuda_zsmul,
 
 	.zmul = cuda_zmul,
 	.zdiv = cuda_zdiv,
@@ -349,6 +350,7 @@ const struct vec_ops gpu_ops = {
 	.zconj = cuda_zconj,
 	.zexpj = cuda_zexpj,
 	.zarg = cuda_zarg,
+	.zabs = cuda_zabs,
 
 	.zcmp = cuda_zcmp,
 	.zdiv_reg = cuda_zdiv_reg,
