@@ -13,7 +13,6 @@
 
 #include <complex.h>
 #include <stdbool.h>
-#include <stdlib.h>
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
@@ -26,6 +25,7 @@
 #include "misc/types.h"
 #include "misc/debug.h"
 #include "misc/shrdptr.h"
+#include "misc/nested.h"
 
 #include "ops.h"
 
@@ -300,7 +300,7 @@ static void op_p_del(const operator_data_t* _data)
 {
 	const struct op_p_data_s* data = CAST_DOWN(op_p_data_s, _data);
 	data->del(data->data);
-	free((void*)data);
+	xfree(data);
 }
 
 operator_data_t* operator_p_get_data(const struct operator_p_s* _data)
@@ -398,7 +398,7 @@ static void identity_free(const operator_data_t* _data)
         const struct identity_s* d = CAST_DOWN(identity_s, _data);
         iovec_free(d->domain);
         iovec_free(d->codomain);
-	free((void*)d);
+	xfree(d);
 }
 
 
@@ -465,7 +465,7 @@ static void chain_free(const operator_data_t* _data)
 	operator_free(data->a);
 	operator_free(data->b);
 
-	free((void*)data);
+	xfree(data);
 }
 
 
@@ -549,7 +549,7 @@ static void stack_free(const operator_data_t* _data)
 	operator_free(data->a);
 	operator_free(data->b);
 
-	free((void*)data);
+	xfree(data);
 }
 
 static bool stack_compatible(unsigned int D, const struct iovec_s* a, const struct iovec_s* b)
@@ -777,27 +777,27 @@ static void op_loop_del(const operator_data_t* _data)
 
 	for (unsigned int i = 0; i < data->N; i++) {
 
-		free((void*)data->dims[i]);
-		free((void*)data->strs[i]);
+		xfree(data->dims[i]);
+		xfree(data->strs[i]);
 	}
 
-	free((void*)data->strs);
-	free((void*)data->dims);
-	free((void*)data->dims0);
-	free((void*)data);
-}
-
-static void op_loop_nary(void* _data, void* ptr[])
-{
-	const struct op_loop_s* data = _data;
-	operator_generic_apply_unchecked(data->op, data->N, ptr);
+	xfree(data->strs);
+	xfree(data->dims);
+	xfree(data->dims0);
+	xfree(data);
 }
 
 static void op_loop_fun(const operator_data_t* _data, unsigned int N, void* args[N])
 {
 	const struct op_loop_s* data = CAST_DOWN(op_loop_s, _data);
 	assert(N == data->N);
-	md_nary(N, data->D, data->dims0, data->strs, args, (void*)data, op_loop_nary);
+
+	NESTED(void, op_loop_nary, (void* ptr[]))
+	{
+		operator_generic_apply_unchecked(data->op, data->N, ptr);
+	};
+
+	md_nary(N, data->D, data->dims0, data->strs, args, op_loop_nary);
 }
 
 static void merge_dims(unsigned int D, long odims[D], const long idims1[D], const long idims2[D])
@@ -1037,7 +1037,7 @@ static void gpuwrp_del(const operator_data_t* _data)
 
 	operator_free(data->op);
 
-	free((void*)data);
+	xfree(data);
 }
 
 const struct operator_s* operator_gpu_wrapper(const struct operator_s* op)
