@@ -25,7 +25,6 @@ struct vec_ops {
 	void (*sqrt)(long N, float* dst, const float* src);
 
 	void (*le)(long N, float* dst, const float* src1, const float* src2);
-	void (*ge)(long N, float* dst, const float* src1, const float* src2);
 
 	void (*add)(long N, float* dst, const float* src1, const float* src2);
 	void (*sub)(long N, float* dst, const float* src1, const float* src2);
@@ -33,6 +32,7 @@ struct vec_ops {
 	void (*div)(long N, float* dst, const float* src1, const float* src2);
 	void (*fmac)(long N, float* dst, const float* src1, const float* src2);
 	void (*fmac2)(long N, double* dst, const float* src1, const float* src2);
+	void (*smul)(long N, float alpha, float* dst, const float* src1);
 
 	void (*zmul)(long N, _Complex float* dst, const _Complex float* src1, const _Complex float* src2);
 	void (*zdiv)(long N, _Complex float* dst, const _Complex float* src1, const _Complex float* src2);
@@ -42,11 +42,14 @@ struct vec_ops {
 	void (*zfmacc)(long N, _Complex float* dst, const _Complex float* src1, const _Complex float* src2);
 	void (*zfmacc2)(long N, _Complex double* dst, const _Complex float* src1, const _Complex float* src2);
 
+	void (*zsmul)(long N, _Complex float val, _Complex float* dst, const _Complex float* src1);
+
 	void (*zpow)(long N,  _Complex float* dst, const _Complex float* src1, const _Complex float* src2);
 	void (*zphsr)(long N, _Complex float* dst, const _Complex float* src);
 	void (*zconj)(long N, _Complex float* dst, const _Complex float* src);
 	void (*zexpj)(long N, _Complex float* dst, const _Complex float* src);
 	void (*zarg)(long N, _Complex float* dst, const _Complex float* src);
+	void (*zabs)(long N, _Complex float* dst, const _Complex float* src);
 
 	void (*zcmp)(long N, _Complex float* dst, const _Complex float* src1, const _Complex float* src2);
 	void (*zdiv_reg)(long N, _Complex float* dst, const _Complex float* src1, const _Complex float* src2, _Complex float lambda);
