@@ -28,7 +28,7 @@ static const char help_str[] = "Multiply input1 with scale factor and add input2
 
 int main_saxpy(int argc, char* argv[])
 {
-	mini_cmdline(argc, argv, 4, usage_str, help_str);
+	mini_cmdline(&argc, argv, 4, usage_str, help_str);
 
 	num_init();
 
